@@ -323,7 +323,8 @@ impl PrettyPrint for TypeAnnotation {
 
 pub enum TypeExpression {
     Unity(Span),
-    TypeIdentifier(Span, CompactString),
+    /// A type identifier, optionally with type arguments like `SomeStruct<A, B>`
+    TypeIdentifier(Span, CompactString, Vec<TypeAnnotation>),
     Multiply(Span, Box<TypeExpression>, Box<TypeExpression>),
     Divide(Span, Box<TypeExpression>, Box<TypeExpression>),
     Power(
@@ -338,7 +339,7 @@ impl TypeExpression {
     pub fn full_span(&self) -> Span {
         match self {
             TypeExpression::Unity(s) => *s,
-            TypeExpression::TypeIdentifier(s, _) => *s,
+            TypeExpression::TypeIdentifier(s, _, _) => *s,
             TypeExpression::Multiply(span_op, lhs, rhs) => {
                 span_op.extend(&lhs.full_span()).extend(&rhs.full_span())
             }
@@ -368,8 +369,19 @@ impl PrettyPrint for TypeExpression {
     fn pretty_print(&self) -> Markup {
         match self {
             TypeExpression::Unity(_) => m::type_identifier("1"),
-            TypeExpression::TypeIdentifier(_, ident) => {
-                m::type_identifier(ident.to_compact_string())
+            TypeExpression::TypeIdentifier(_, ident, type_args) => {
+                let mut markup = m::type_identifier(ident.to_compact_string());
+                if !type_args.is_empty() {
+                    markup = markup + m::operator("<");
+                    for (i, arg) in type_args.iter().enumerate() {
+                        if i > 0 {
+                            markup = markup + m::operator(",") + m::space();
+                        }
+                        markup = markup + arg.pretty_print();
+                    }
+                    markup = markup + m::operator(">");
+                }
+                markup
             }
             TypeExpression::Multiply(_, lhs, rhs) => {
                 lhs.pretty_print() + m::space() + m::operator("×") + m::space() + rhs.pretty_print()
@@ -409,7 +421,7 @@ impl ProcedureKind {
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub enum TypeParameterBound {
     Dim,
 }
@@ -456,6 +468,7 @@ pub enum Statement<'a> {
     DefineStruct {
         struct_name_span: Span,
         struct_name: &'a str,
+        type_parameters: Vec<(Span, &'a str, Option<TypeParameterBound>)>,
         fields: Vec<(Span, &'a str, TypeAnnotation)>,
     },
 }
@@ -488,9 +501,11 @@ impl ReplaceSpans for TypeExpression {
     fn replace_spans(&self) -> Self {
         match self {
             TypeExpression::Unity(_) => TypeExpression::Unity(Span::dummy()),
-            TypeExpression::TypeIdentifier(_, d) => {
-                TypeExpression::TypeIdentifier(Span::dummy(), d.clone())
-            }
+            TypeExpression::TypeIdentifier(_, name, type_args) => TypeExpression::TypeIdentifier(
+                Span::dummy(),
+                name.clone(),
+                type_args.iter().map(|a| a.replace_spans()).collect(),
+            ),
             TypeExpression::Multiply(_, lhs, rhs) => TypeExpression::Multiply(
                 Span::dummy(),
                 Box::new(lhs.replace_spans()),
@@ -690,11 +705,16 @@ impl ReplaceSpans for Statement<'_> {
             }
             Statement::DefineStruct {
                 struct_name,
+                type_parameters,
                 fields,
                 ..
             } => Statement::DefineStruct {
                 struct_name_span: Span::dummy(),
                 struct_name,
+                type_parameters: type_parameters
+                    .iter()
+                    .map(|(_, name, bound)| (Span::dummy(), *name, bound.clone()))
+                    .collect(),
                 fields: fields
                     .iter()
                     .map(|(_span, name, type_)| (Span::dummy(), *name, type_.replace_spans()))
