@@ -1,11 +1,12 @@
-use super::macros::*;
 use super::Args;
 use super::Result;
+use super::macros::*;
 
+use crate::interpreter::RuntimeErrorKind;
 use crate::quantity::Quantity;
 use crate::value::Value;
 
-pub fn mod_(mut args: Args) -> Result<Value> {
+pub fn mod_(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let x = quantity_arg!(args);
     let y = quantity_arg!(args);
 
@@ -18,14 +19,14 @@ pub fn mod_(mut args: Args) -> Result<Value> {
 // A simple math function with signature 'Fn[(Scalar) -> Scalar]'
 macro_rules! simple_scalar_math_function {
     ($name:ident, $op:ident) => {
-        pub fn $name(mut args: Args) -> Result<Value> {
+        pub fn $name(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
             let value = scalar_arg!(args).to_f64();
             return_scalar!(value.$op())
         }
     };
 }
 
-pub fn abs(mut args: Args) -> Result<Value> {
+pub fn abs(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let arg = quantity_arg!(args);
     return_quantity!(arg.unsafe_value().to_f64().abs(), arg.unit().clone())
 }
@@ -43,7 +44,7 @@ simple_scalar_math_function!(asin, asin);
 simple_scalar_math_function!(acos, acos);
 simple_scalar_math_function!(atan, atan);
 
-pub fn atan2(mut args: Args) -> Result<Value> {
+pub fn atan2(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let y = quantity_arg!(args);
     let x = quantity_arg!(args);
 
@@ -64,24 +65,24 @@ simple_scalar_math_function!(ln, ln);
 simple_scalar_math_function!(log10, log10);
 simple_scalar_math_function!(log2, log2);
 
-pub fn gamma(mut args: Args) -> Result<Value> {
+pub fn gamma(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let input = scalar_arg!(args).to_f64();
 
     return_scalar!(crate::gamma::gamma(input))
 }
 
-pub fn is_nan(mut args: Args) -> Result<Value> {
+pub fn is_nan(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let arg = quantity_arg!(args);
 
     return_boolean!(arg.unsafe_value().to_f64().is_nan())
 }
 
-pub fn is_infinite(mut args: Args) -> Result<Value> {
+pub fn is_infinite(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let arg = quantity_arg!(args);
 
     return_boolean!(arg.unsafe_value().to_f64().is_infinite())
 }
 
-pub fn random(_args: Args) -> Result<Value> {
+pub fn random(_args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     return_scalar!(rand::random::<f64>())
 }
