@@ -24,6 +24,7 @@ pub mod module_importer;
 mod name_resolution;
 mod number;
 mod parser;
+#[cfg(feature = "plotting")]
 mod plot;
 mod prefix;
 mod prefix_parser;
@@ -64,6 +65,7 @@ use markup::FormatType;
 use markup::Markup;
 use module_importer::{ModuleImporter, NullImporter};
 use prefix_transformer::Transformer;
+use pretty_print::PrettyPrint;
 
 use resolver::CodeSource;
 use resolver::Resolver;
@@ -74,7 +76,7 @@ use typechecker::{TypeCheckError, TypeChecker};
 pub use diagnostic::Diagnostic;
 pub use interpreter::InterpreterResult;
 pub use interpreter::InterpreterSettings;
-pub use interpreter::RuntimeError;
+pub use interpreter::{RuntimeError, RuntimeErrorKind};
 pub use name_resolution::NameResolutionError;
 pub use parser::ParseError;
 pub use registry::BaseRepresentation;
@@ -113,6 +115,16 @@ pub struct Context {
     terminal_width: Option<usize>,
 }
 
+pub struct FunctionInfo {
+    pub fn_name: CompactString,
+    pub name: Option<CompactString>,
+    pub signature_str: CompactString,
+    pub description: Option<CompactString>,
+    pub url: Option<CompactString>,
+    pub examples: Vec<(CompactString, Option<CompactString>)>,
+    pub code_source: CodeSource,
+}
+
 impl Context {
     pub fn new(module_importer: impl ModuleImporter + 'static) -> Self {
         Context {
@@ -150,6 +162,10 @@ impl Context {
         ExchangeRatesCache::use_test_rates();
     }
 
+    pub fn runtime_error(&self, kind: RuntimeErrorKind) -> RuntimeError {
+        self.interpreter.runtime_error(kind)
+    }
+
     pub fn variable_names(&self) -> impl Iterator<Item = CompactString> + '_ {
         self.prefix_transformer
             .variable_names
@@ -166,37 +182,26 @@ impl Context {
             .cloned()
     }
 
-    pub fn functions(
-        &self,
-    ) -> impl Iterator<
-        Item = (
-            CompactString,
-            Option<CompactString>,
-            CompactString,
-            Option<CompactString>,
-            Option<CompactString>,
-            Vec<(CompactString, Option<CompactString>)>,
-            CodeSource,
-        ),
-    > + '_ {
+    pub fn functions(&self) -> impl Iterator<Item = FunctionInfo> + '_ {
         self.prefix_transformer
             .function_names
             .iter()
             .filter(|name| !name.starts_with('_'))
             .map(move |name| {
                 let (signature, meta) = self.typechecker.lookup_function(name).unwrap();
-                (
-                    name.clone(),
-                    meta.name.clone(),
-                    signature
+                FunctionInfo {
+                    fn_name: name.clone(),
+                    name: meta.name.clone(),
+                    signature_str: signature
                         .pretty_print(self.dimension_registry())
                         .to_compact_string(),
-                    meta.description.clone(),
-                    meta.url.clone(),
-                    meta.examples.clone(),
-                    self.resolver
+                    description: meta.description.clone(),
+                    url: meta.url.clone(),
+                    examples: meta.examples.clone(),
+                    code_source: self
+                        .resolver
                         .get_code_source(signature.definition_span.code_source_id),
-                )
+                }
             })
     }
 
@@ -233,19 +238,19 @@ impl Context {
         formatter.format(entries, format_type)
     }
 
-    pub fn print_functions(&self) -> Markup {
+    fn print_functions(&self) -> Markup {
         self.print_sorted(self.function_names().collect(), FormatType::Identifier)
     }
 
-    pub fn print_dimensions(&self) -> Markup {
+    fn print_dimensions(&self) -> Markup {
         self.print_sorted(self.dimension_names().into(), FormatType::TypeIdentifier)
     }
 
-    pub fn print_variables(&self) -> Markup {
+    fn print_variables(&self) -> Markup {
         self.print_sorted(self.variable_names().collect(), FormatType::Identifier)
     }
 
-    pub fn print_units(&self) -> Markup {
+    fn print_units(&self) -> Markup {
         let units = self.unit_names().iter().flatten().cloned().collect();
         self.print_sorted(units, FormatType::Unit)
     }
@@ -292,9 +297,18 @@ impl Context {
             add_if_valid(variable.into());
         }
 
-        for mut function in self.function_names() {
+        for function_name in self.function_names() {
+            let mut function = function_name.clone();
             if add_paren {
-                function.push('(');
+                if let Some((signature, _)) = self.typechecker.lookup_function(&function_name) {
+                    if signature.parameters.is_empty() {
+                        function.push_str("()");
+                    } else {
+                        function.push('(');
+                    }
+                } else {
+                    function.push('(');
+                }
             }
             add_if_valid(function.into());
         }
@@ -344,107 +358,177 @@ impl Context {
         }
         let reg = self.interpreter.get_unit_registry();
 
+        // Check if it's a unit
         if let PrefixParserResult::UnitIdentifier(_span, prefix, _, full_name) =
             self.prefix_transformer.prefix_parser.parse(keyword)
-        {
-            if let Some(md) = reg
+            && let Some(md) = reg
                 .inner
                 .get_base_representation_for_name(&full_name)
                 .ok()
                 .map(|(_, md)| md)
-            {
-                let mut help = m::text("Unit: ")
-                    + m::unit(md.name.unwrap_or_else(|| keyword.to_compact_string()));
-                if let Some(url) = &md.url {
-                    help += m::text(" (") + m::string(url_encode(url)) + m::text(")");
-                }
-                help += m::nl();
-                if md.aliases.len() > 1 {
-                    help += m::text("Aliases: ")
-                        + m::text(
-                            md.aliases
-                                .iter()
-                                .map(|(x, _)| x.as_str())
-                                .collect::<Vec<_>>()
-                                .join_compact(", "),
-                        )
-                        + m::nl();
-                }
-
-                if let Some(description) = &md.description {
-                    let desc = "Description: ";
-                    let mut lines = description.lines();
-                    help += m::text(desc)
-                        + m::text(
-                            lines
-                                .by_ref()
-                                .next()
-                                .unwrap_or("")
-                                .trim()
-                                .to_compact_string(),
-                        )
-                        + m::nl();
-
-                    for line in lines {
-                        help += m::whitespace(CompactString::const_new(" ").repeat(desc.len()))
-                            + m::text(line.trim().to_compact_string())
-                            + m::nl();
-                    }
-                }
-
-                if matches!(md.type_, Type::Dimension(d) if d.is_scalar()) {
-                    help += m::text("A dimensionless unit ([")
-                        + md.readable_type
-                        + m::text("])")
-                        + m::nl();
-                } else {
-                    help += m::text("A unit of: ") + md.readable_type + m::nl();
-                }
-
-                if let Some(defining_info) = self.interpreter.get_defining_unit(&full_name) {
-                    let x = defining_info
-                        .iter()
-                        .filter(|u| !u.unit_id.is_base())
-                        .map(|unit_factor| unit_factor.unit_id.unit_and_factor())
-                        .next();
-
-                    if !prefix.is_none() {
-                        help += m::nl()
-                            + m::value("1 ")
-                            + m::unit(keyword.to_compact_string())
-                            + m::text(" = ")
-                            + m::value(prefix.factor().pretty_print())
-                            + m::space()
-                            + m::unit(full_name.to_compact_string());
-                    }
-
-                    if let Some(BaseUnitAndFactor(prod, num)) = x {
-                        help += m::nl()
-                            + m::value("1 ")
-                            + m::unit(full_name.to_compact_string())
-                            + m::text(" = ")
-                            + m::value(num.pretty_print())
-                            + m::space()
-                            + prod.pretty_print_with(
-                                |f| f.exponent,
-                                'x',
-                                '/',
-                                true,
-                                Some(m::FormatType::Unit),
-                            );
-                    } else {
-                        help += m::nl()
-                            + m::unit(full_name.to_compact_string())
-                            + m::text(" is a base unit");
-                    }
-                };
-
-                help += m::nl();
-
-                return help;
+        {
+            let mut help =
+                m::text("Unit: ") + m::unit(md.name.unwrap_or_else(|| keyword.to_compact_string()));
+            if let Some(url) = &md.url {
+                help += m::text(" (") + m::string(url_encode(url)) + m::text(")");
             }
+            help += m::nl();
+            if md.aliases.len() > 1 {
+                help += m::text("Aliases: ")
+                    + m::text(
+                        md.aliases
+                            .iter()
+                            .map(|(x, _)| x.as_str())
+                            .collect::<Vec<_>>()
+                            .join_compact(", "),
+                    )
+                    + m::nl();
+            }
+
+            if let Some(description) = &md.description {
+                let desc = "Description: ";
+                let mut lines = description.lines();
+                help += m::text(desc)
+                    + m::text(
+                        lines
+                            .by_ref()
+                            .next()
+                            .unwrap_or("")
+                            .trim()
+                            .to_compact_string(),
+                    )
+                    + m::nl();
+
+                for line in lines {
+                    help += m::whitespace(CompactString::const_new(" ").repeat(desc.len()))
+                        + m::text(line.trim().to_compact_string())
+                        + m::nl();
+                }
+            }
+
+            if matches!(md.type_, Type::Dimension(d) if d.is_scalar()) {
+                help +=
+                    m::text("A dimensionless unit ([") + md.readable_type + m::text("])") + m::nl();
+            } else {
+                help += m::text("A unit of: ") + md.readable_type + m::nl();
+            }
+
+            if let Some(defining_info) = self.interpreter.get_defining_unit(&full_name) {
+                let x = defining_info
+                    .iter()
+                    .filter(|u| !u.unit_id.is_base())
+                    .map(|unit_factor| unit_factor.unit_id.unit_and_factor())
+                    .next();
+
+                if !prefix.is_none() {
+                    help += m::nl()
+                        + m::value("1 ")
+                        + m::unit(keyword.to_compact_string())
+                        + m::text(" = ")
+                        + m::value(prefix.factor().pretty_print())
+                        + m::space()
+                        + m::unit(full_name.to_compact_string());
+                }
+
+                if let Some(BaseUnitAndFactor(prod, num)) = x {
+                    help += m::nl()
+                        + m::value("1 ")
+                        + m::unit(full_name.to_compact_string())
+                        + m::text(" = ")
+                        + m::value(num.pretty_print())
+                        + m::space()
+                        + prod.pretty_print_with(
+                            |f| f.exponent,
+                            'x',
+                            '/',
+                            true,
+                            Some(m::FormatType::Unit),
+                        );
+                } else {
+                    help += m::nl()
+                        + m::unit(full_name.to_compact_string())
+                        + m::text(" is a base unit");
+                }
+            };
+
+            help += m::nl();
+
+            return help;
         };
 
+        // Check if it's a dimension
+        if self.dimension_registry().contains(keyword) {
+            let mut help =
+                m::text("Dimension:   ") + m::type_identifier(keyword.to_compact_string());
+
+            if let Ok(base_representation) = self
+                .dimension_registry()
+                .get_base_representation_for_name(keyword)
+            {
+                if self.dimension_registry().is_base_dimension(keyword) {
+                    help += m::text(" (Base dimension)") + m::nl();
+                } else {
+                    help += m::space()
+                        + m::operator("=")
+                        + m::space()
+                        + base_representation.pretty_print()
+                        + m::nl();
+                }
+
+                let equivalent_dimensions = self
+                    .dimension_registry()
+                    .get_derived_entry_names_for(&base_representation);
+                if equivalent_dimensions.len() > 1 {
+                    let other_names: Vec<CompactString> = equivalent_dimensions
+                        .iter()
+                        .filter(|&name| name.as_str() != keyword)
+                        .cloned()
+                        .collect();
+                    if !other_names.is_empty() {
+                        help += m::text("Equivalent:  ");
+                        for (i, name) in other_names.iter().enumerate() {
+                            if i > 0 {
+                                help += m::text(", ");
+                            }
+                            help += m::type_identifier(name.clone());
+                        }
+                        help += m::nl();
+                    }
+                }
+
+                // List units that belong to this dimension
+                let matching_units: Vec<CompactString> = self
+                    .unit_representations()
+                    .filter_map(|(_unit_name, (_unit_base_rep, unit_metadata))| {
+                        if let Type::Dimension(unit_dim) = &unit_metadata.type_
+                            && unit_dim.to_base_representation() == base_representation
+                            && let Some((primary_alias, _)) = unit_metadata.aliases.first()
+                        {
+                            return Some(primary_alias.clone());
+                        }
+                        None
+                    })
+                    .collect();
+
+                if !matching_units.is_empty() {
+                    let mut sorted_units = matching_units;
+                    sorted_units.sort_by_key(|unit| unit.to_lowercase());
+
+                    help += m::text("Units:       ");
+                    for (i, unit) in sorted_units.iter().enumerate() {
+                        if i > 0 {
+                            help += m::text(", ");
+                        }
+                        help += m::unit(unit.clone());
+                    }
+                    help += m::nl();
+                }
+            }
+
+            return help;
+        }
+
+        // Check if it's a valid identifier
         if let Some(l) = self.interpreter.lookup_global(keyword) {
             let mut help = m::text("Variable: ");
             if let Some(name) = &l.metadata.name {
@@ -498,6 +582,7 @@ impl Context {
             return help;
         }
 
+        // Check if it's a function
         if let Some((fn_signature, fn_metadata)) = self.typechecker.lookup_function(keyword) {
             let metadata = fn_metadata.clone();
 
@@ -649,198 +734,50 @@ impl Context {
             self.prefix_transformer = prefix_transformer_old.clone();
             self.typechecker = typechecker_old.clone();
 
-            if self.load_currency_module_on_demand {
-                if let Err(NumbatError::TypeCheckError(TypeCheckError::UnknownIdentifier(
+            if self.load_currency_module_on_demand
+                && let Err(NumbatError::TypeCheckError(TypeCheckError::UnknownIdentifier(
                     _,
                     identifier,
                     _,
                 ))) = &result
-                {
-                    // TODO: maybe we can somehow load this list of identifiers from units::currencies?
-                    const CURRENCY_IDENTIFIERS: &[&str] = &[
-                        "$",
-                        "USD",
-                        "usd",
-                        "dollar",
-                        "dollars",
-                        "A$",
-                        "AUD",
-                        "aud",
-                        "australian_dollar",
-                        "australian_dollars",
-                        "C$",
-                        "CAD",
-                        "cad",
-                        "canadian_dollar",
-                        "canadian_dollars",
-                        "CHF",
-                        "chf",
-                        "swiss_franc",
-                        "swiss_francs",
-                        "CNY",
-                        "cny",
-                        "yuan",
-                        "renminbi",
-                        "元",
-                        "EUR",
-                        "eur",
-                        "euro",
-                        "euros",
-                        "€",
-                        "GBP",
-                        "gbp",
-                        "british_pound",
-                        "pound_sterling",
-                        "£",
-                        "JPY",
-                        "jpy",
-                        "yen",
-                        "yens",
-                        "¥",
-                        "円",
-                        "bulgarian_lev",
-                        "bulgarian_leva",
-                        "BGN",
-                        "bgn",
-                        "czech_koruna",
-                        "czech_korunas",
-                        "CZK",
-                        "czk",
-                        "Kč",
-                        "hungarian_forint",
-                        "hungarian_forints",
-                        "HUF",
-                        "huf",
-                        "Ft",
-                        "polish_zloty",
-                        "polish_zlotys",
-                        "PLN",
-                        "pln",
-                        "zł",
-                        "romanian_leu",
-                        "romanian_leus",
-                        "RON",
-                        "ron",
-                        "lei",
-                        "turkish_lira",
-                        "turkish_liras",
-                        "TRY",
-                        "try",
-                        "₺",
-                        "brazilian_real",
-                        "brazilian_reals",
-                        "BRL",
-                        "brl",
-                        "R$",
-                        "hong_kong_dollar",
-                        "hong_kong_dollars",
-                        "HKD",
-                        "hkd",
-                        "HK$",
-                        "hk$",
-                        "indonesian_rupiah",
-                        "indonesian_rupiahs",
-                        "IDR",
-                        "idr",
-                        "Rp",
-                        "indian_rupee",
-                        "indian_rupees",
-                        "INR",
-                        "inr",
-                        "₹",
-                        "south_korean_won",
-                        "south_korean_wons",
-                        "KRW",
-                        "krw",
-                        "₩",
-                        "malaysian_ringgit",
-                        "malaysian_ringgits",
-                        "MYR",
-                        "RM",
-                        "new_zealand_dollar",
-                        "new_zealand_dollars",
-                        "NZD",
-                        "nzd",
-                        "NZ$",
-                        "nz$",
-                        "philippine_peso",
-                        "philippine_pesos",
-                        "PHP",
-                        "php",
-                        "₱",
-                        "singapore_dollar",
-                        "singapore_dollars",
-                        "SGD",
-                        "sgd",
-                        "S$",
-                        "thai_baht",
-                        "thai_bahts",
-                        "THB",
-                        "thb",
-                        "฿",
-                        "danish_krone",
-                        "danish_kroner",
-                        "DKK",
-                        "dkk",
-                        "swedish_krona",
-                        "swedish_kronor",
-                        "SEK",
-                        "sek",
-                        "icelandic_króna",
-                        "icelandic_krónur",
-                        "ISK",
-                        "isk",
-                        "norwegian_krone",
-                        "norwegian_kroner",
-                        "NOK",
-                        "nok",
-                        "israeli_new_shekel",
-                        "israeli_new_shekels",
-                        "ILS",
-                        "ils",
-                        "₪",
-                        "NIS",
-                        "nis",
-                        "south_african_rand",
-                        "ZAR",
-                        "zar",
-                    ];
-                    if CURRENCY_IDENTIFIERS.contains(&identifier.as_str()) {
-                        let mut no_print_settings = InterpreterSettings {
-                            print_fn: Box::new(
-                                move |_: &m::Markup| { // ignore any print statements when loading this module asynchronously
-                                },
-                            ),
-                        };
+            {
+                const CURRENCY_IDENTIFIERS: &[&str] =
+                    &include!(concat!(env!("OUT_DIR"), "/currencies.rs"));
+                if CURRENCY_IDENTIFIERS.contains(&identifier.as_str()) {
+                    let mut no_print_settings = InterpreterSettings {
+                        print_fn: Box::new(
+                            move |_: &m::Markup| { // ignore any print statements when loading this module asynchronously
+                            },
+                        ),
+                    };
 
-                        // We also call this from a thread at program startup, so if a user only starts
-                        // to use currencies later on, this will already be available and return immediately.
-                        // Otherwise, we fetch it now and make sure to block on this call.
-                        {
-                            let erc = ExchangeRatesCache::fetch();
+                    // We also call this from a thread at program startup, so if a user only starts
+                    // to use currencies later on, this will already be available and return immediately.
+                    // Otherwise, we fetch it now and make sure to block on this call.
+                    {
+                        let erc = ExchangeRatesCache::fetch();
 
-                            if erc.is_none() {
-                                return Err(Box::new(NumbatError::RuntimeError(
-                                    RuntimeError::CouldNotLoadExchangeRates,
-                                )));
-                            }
+                        if erc.is_none() {
+                            return Err(Box::new(NumbatError::RuntimeError(
+                                self.runtime_error(RuntimeErrorKind::CouldNotLoadExchangeRates),
+                            )));
                         }
-
-                        let _ = self.interpret_with_settings(
-                            &mut no_print_settings,
-                            "use units::currencies",
-                            CodeSource::Internal,
-                        )?;
-
-                        // Make sure we do not run into an infinite loop in case loading that
-                        // module did not bring in the required currency unit identifier. This
-                        // can happen if the list of currency identifiers is not in sync with
-                        // what the module actually defines.
-                        self.load_currency_module_on_demand = false;
-
-                        // Now we try to evaluate the user expression again:
-                        return self.interpret_with_settings(settings, code, code_source);
                     }
+
+                    let _ = self.interpret_with_settings(
+                        &mut no_print_settings,
+                        "use units::currencies",
+                        CodeSource::Internal,
+                    )?;
+
+                    // Make sure we do not run into an infinite loop in case loading that
+                    // module did not bring in the required currency unit identifier. This
+                    // can happen if the list of currency identifiers is not in sync with
+                    // what the module actually defines.
+                    self.load_currency_module_on_demand = false;
+
+                    // Now we try to evaluate the user expression again:
+                    return self.interpret_with_settings(settings, code, code_source);
                 }
             }
         }
@@ -876,14 +813,19 @@ impl Context {
         Ok((typed_statements, result))
     }
 
-    pub fn print_diagnostic(&self, error: impl ErrorDiagnostic) {
+    pub fn print_diagnostic(&self, error: impl ErrorDiagnostic, colorize: bool) {
         use codespan_reporting::term::{
-            self,
+            self, Config,
             termcolor::{ColorChoice, StandardStream},
-            Config,
         };
 
-        let writer = StandardStream::stderr(ColorChoice::Auto);
+        let color = if colorize {
+            ColorChoice::Auto
+        } else {
+            ColorChoice::Never
+        };
+
+        let writer = StandardStream::stderr(color);
         let config = Config::default();
 
         // we want to be sure no one can write between our diagnostics
