@@ -1,24 +1,26 @@
+use compact_str::CompactString;
+
+use crate::Type;
 use crate::ast::{TypeAnnotation, TypeParameterBound};
 use crate::dimension::DimensionRegistry;
 use crate::pretty_print::PrettyPrint;
 use crate::span::Span;
 use crate::type_variable::TypeVariable;
 use crate::typed_ast::pretty_print_function_signature;
-use crate::Type;
 
 use super::map_stack::MapStack;
 use super::substitutions::{ApplySubstitution, Substitution, SubstitutionError};
 use super::type_scheme::TypeScheme;
 
-type Identifier = String;
+type Identifier = CompactString;
 
 #[derive(Clone, Debug)]
 pub struct FunctionSignature {
-    pub name: String,
+    pub name: CompactString,
     pub definition_span: Span,
     #[allow(dead_code)]
-    pub type_parameters: Vec<(Span, String, Option<TypeParameterBound>)>,
-    pub parameters: Vec<(Span, String, Option<TypeAnnotation>)>,
+    pub type_parameters: Vec<(Span, CompactString, Option<TypeParameterBound>)>,
+    pub parameters: Vec<(Span, CompactString, Option<TypeAnnotation>)>,
     pub return_type_annotation: Option<TypeAnnotation>,
     pub fn_type: TypeScheme,
 }
@@ -64,10 +66,10 @@ impl FunctionSignature {
 
 #[derive(Clone, Debug)]
 pub struct FunctionMetadata {
-    pub name: Option<String>,
-    pub url: Option<String>,
-    pub description: Option<String>,
-    pub examples: Vec<(String, Option<String>)>,
+    pub name: Option<CompactString>,
+    pub url: Option<CompactString>,
+    pub description: Option<CompactString>,
+    pub examples: Vec<(CompactString, Option<CompactString>)>,
 }
 
 #[derive(Clone, Debug)]
@@ -119,7 +121,7 @@ impl Environment {
 
     pub(crate) fn add_function(
         &mut self,
-        v: String,
+        v: CompactString,
         signature: FunctionSignature,
         metadata: FunctionMetadata,
     ) {
@@ -181,6 +183,18 @@ impl Environment {
             }
         }
     }
+
+    pub(crate) fn get_proper_function_reference<'a>(
+        &self,
+        expr: &crate::ast::Expression<'a>,
+    ) -> Option<(&'a str, &FunctionSignature)> {
+        match expr {
+            crate::ast::Expression::Identifier(_, name) => self
+                .get_function_info(name)
+                .map(|(signature, _)| (*name, signature)),
+            _ => None,
+        }
+    }
 }
 
 impl ApplySubstitution for Environment {
