@@ -27,7 +27,7 @@ use crate::name_resolution::Namespace;
 use crate::pretty_print::PrettyPrint;
 use crate::span::Span;
 use crate::type_variable::TypeVariable;
-use crate::typed_ast::{self, DType, DTypeFactor, Expression, StructInfo, Type};
+use crate::typed_ast::{self, DType, DTypeFactor, Expression, StructInfo, StructKind, Type};
 use crate::{decorator, ffi, suggestion};
 
 use compact_str::{CompactString, ToCompactString, format_compact};
@@ -235,20 +235,65 @@ impl TypeChecker {
     fn type_from_annotation(&self, annotation: &TypeAnnotation) -> Result<Type> {
         match annotation {
             TypeAnnotation::TypeExpression(dexpr) => {
-                if let TypeExpression::TypeIdentifier(_, name) = dexpr
-                    && let Some(info) = self.structs.get(name)
+                if let TypeExpression::TypeIdentifier(span, name, type_args) = dexpr
+                    && let Some(struct_info) = self.structs.get(name)
                 {
-                    // if we see a struct name here, it's safe to assume it
-                    // isn't accidentally clashing with a dimension, we
-                    // check that earlier.
-                    return Ok(Type::Struct(Box::new(info.clone())));
+                    // Get the type parameters from the struct definition
+                    let type_parameters = match &struct_info.kind {
+                        StructKind::Definition(params) => params,
+                        StructKind::Instance(_) => {
+                            unreachable!("Struct registry should only contain definitions")
+                        }
+                    };
+
+                    // Check that the number of type arguments matches the number of type parameters
+                    if type_args.len() != type_parameters.len() {
+                        return Err(Box::new(TypeCheckError::WrongNumberOfTypeArguments {
+                            span: *span,
+                            type_name: name.to_string(),
+                            expected: type_parameters.len(),
+                            actual: type_args.len(),
+                        }));
+                    }
+
+                    if type_args.is_empty() {
+                        // Non-generic struct: create an instance with empty type arguments
+                        return Ok(Type::Struct(Box::new(StructInfo {
+                            definition_span: struct_info.definition_span,
+                            name: struct_info.name.clone(),
+                            kind: StructKind::Instance(vec![]),
+                            fields: struct_info.fields.clone(),
+                        })));
+                    }
+
+                    // Build substitution from type parameters to type arguments
+                    let mut substitution = Substitution::empty();
+                    let mut concrete_type_args = Vec::new();
+                    for ((_, param_name, _), arg) in type_parameters.iter().zip(type_args.iter()) {
+                        let arg_type = self.type_from_annotation(arg)?;
+                        concrete_type_args.push(arg_type.clone());
+                        substitution.append(TypeVariable::new(param_name), arg_type);
+                    }
+
+                    // Create instantiated struct with substituted field types
+                    let mut instantiated_fields = struct_info.fields.clone();
+                    for (_, field_type) in instantiated_fields.values_mut() {
+                        field_type.apply(&substitution).ok();
+                    }
+
+                    return Ok(Type::Struct(Box::new(StructInfo {
+                        definition_span: struct_info.definition_span,
+                        name: struct_info.name.clone(),
+                        kind: StructKind::Instance(concrete_type_args),
+                        fields: instantiated_fields,
+                    })));
                 }
 
                 let mut factors = self
                     .registry
                     .get_base_representation(dexpr)
                     .map(DType::from)
-                    .map_err(TypeCheckError::RegistryError)?
+                    .map_err(TypeCheckError::from)?
                     .into_factors();
 
                 // Replace BaseDimension("D") with TVar("D") for all type parameters
@@ -953,6 +998,63 @@ impl TypeChecker {
                     )));
                 };
 
+                // For generic structs, instantiate type parameters with fresh type variables
+                let instantiated_struct_info = match &struct_info.kind {
+                    StructKind::Definition(type_parameters) if type_parameters.is_empty() => {
+                        // Non-generic struct: create instance with empty type arguments
+                        StructInfo {
+                            definition_span: struct_info.definition_span,
+                            name: struct_info.name.clone(),
+                            kind: StructKind::Instance(vec![]),
+                            fields: struct_info.fields.clone(),
+                        }
+                    }
+                    StructKind::Definition(type_parameters) => {
+                        // Generate fresh type variables for each type parameter
+                        let variables: Vec<_> = type_parameters
+                            .iter()
+                            .map(|_| self.name_generator.fresh_type_variable())
+                            .collect();
+
+                        // Build substitution mapping type parameter names to fresh type variables
+                        let substitution = type_parameters.iter().zip(variables.iter()).fold(
+                            Substitution::empty(),
+                            |mut subst, ((_, name, _), variable)| {
+                                subst.append(TypeVariable::new(name), Type::TVar(variable.clone()));
+                                subst
+                            },
+                        );
+
+                        // Add dtype constraints for type parameters with `Dim` bounds
+                        for ((_, _, bound), variable) in
+                            type_parameters.iter().zip(variables.iter())
+                        {
+                            if let Some(TypeParameterBound::Dim) = bound {
+                                self.add_dtype_constraint(&Type::TVar(variable.clone()))
+                                    .ok();
+                            }
+                        }
+
+                        // Create instantiated StructInfo with substituted field types
+                        let mut instantiated_fields = struct_info.fields.clone();
+                        for (_, field_type) in instantiated_fields.values_mut() {
+                            field_type.apply(&substitution).ok();
+                        }
+
+                        StructInfo {
+                            definition_span: struct_info.definition_span,
+                            name: struct_info.name.clone(),
+                            kind: StructKind::Instance(
+                                variables.iter().map(|v| Type::TVar(v.clone())).collect(),
+                            ),
+                            fields: instantiated_fields,
+                        }
+                    }
+                    StructKind::Instance(_) => {
+                        unreachable!("Struct registry should only contain definitions")
+                    }
+                };
+
                 let mut seen_fields = HashMap::new();
 
                 for ((field, expr), span) in
@@ -968,13 +1070,14 @@ impl TypeChecker {
                         ));
                     }
 
-                    let Some((expected_field_span, expected_type)) = struct_info.fields.get(*field)
+                    let Some((expected_field_span, expected_type)) =
+                        instantiated_struct_info.fields.get(*field)
                     else {
                         return Err(Box::new(TypeCheckError::UnknownFieldInStructInstantiation(
                             *span,
-                            struct_info.definition_span,
+                            instantiated_struct_info.definition_span,
                             field.to_string(),
-                            struct_info.name.to_string(),
+                            instantiated_struct_info.name.to_string(),
                         )));
                     };
 
@@ -995,7 +1098,7 @@ impl TypeChecker {
                 }
 
                 let missing_fields = {
-                    let mut fields = struct_info.fields.clone();
+                    let mut fields = instantiated_struct_info.fields.clone();
                     fields.retain(|f, _| !seen_fields.contains_key(&f.as_str()));
                     fields.into_iter().map(|(n, (_, t))| (n, t)).collect_vec()
                 };
@@ -1004,7 +1107,7 @@ impl TypeChecker {
                     return Err(Box::new(
                         TypeCheckError::MissingFieldsInStructInstantiation(
                             *full_span,
-                            struct_info.definition_span,
+                            instantiated_struct_info.definition_span,
                             missing_fields,
                         ),
                     ));
@@ -1013,7 +1116,7 @@ impl TypeChecker {
                 typed_ast::Expression::InstantiateStruct(
                     *full_span,
                     fields_checked,
-                    struct_info.clone(),
+                    instantiated_struct_info,
                 )
             }
             ast::Expression::AccessField(full_span, ident_span, expr, field_name) => {
@@ -1263,7 +1366,7 @@ impl TypeChecker {
                     let dtype: DType = self
                         .registry
                         .get_base_representation(dexpr)
-                        .map_err(TypeCheckError::RegistryError)?
+                        .map_err(TypeCheckError::from)?
                         .into();
 
                     if dtype.is_scalar() {
@@ -1275,13 +1378,16 @@ impl TypeChecker {
 
                     dtype
                 } else {
+                    use crate::dimension::DimensionRegistryError;
                     use heck::ToUpperCamelCase;
                     // In a unit definition like 'unit pixel' without a specified type,
                     // we add a new type for the user
                     let type_name = unit_name.to_upper_camel_case();
                     self.registry
                         .add_base_dimension(&type_name)
-                        .map_err(TypeCheckError::RegistryError)?
+                        .map_err(|err| {
+                            TypeCheckError::from(DimensionRegistryError { span: *span, err })
+                        })?
                         .into()
                 };
                 for (name, _) in decorator::name_and_aliases(unit_name, decorators) {
@@ -1295,6 +1401,7 @@ impl TypeChecker {
 
                 typed_ast::Statement::DefineBaseUnit(
                     unit_name,
+                    *span,
                     decorators.clone(),
                     type_annotation.clone().map(TypeAnnotation::TypeExpression),
                     TypeScheme::concrete(Type::Dimension(type_specified)),
@@ -1389,7 +1496,7 @@ impl TypeChecker {
                             *span,
                             CompactString::const_new("type parameter"),
                         )
-                        .ok(); // TODO: is this call even correct?
+                        .ok();
 
                     self.registry.introduced_type_parameters.push((
                         *span,
@@ -1627,7 +1734,7 @@ impl TypeChecker {
                 if let Some(dexpr) = dexprs.first() {
                     self.registry
                         .add_derived_dimension(name, dexpr)
-                        .map_err(TypeCheckError::RegistryError)?;
+                        .map_err(TypeCheckError::from)?;
 
                     let base_representation = self
                         .registry
@@ -1638,7 +1745,7 @@ impl TypeChecker {
                         let alternative_base_representation = self
                             .registry
                             .get_base_representation(alternative_expr)
-                            .map_err(TypeCheckError::RegistryError)?;
+                            .map_err(TypeCheckError::from)?;
                         if alternative_base_representation != base_representation {
                             return Err(Box::new(
                                 TypeCheckError::IncompatibleAlternativeDimensionExpression(
@@ -1652,9 +1759,13 @@ impl TypeChecker {
                         }
                     }
                 } else {
-                    self.registry
-                        .add_base_dimension(name)
-                        .map_err(TypeCheckError::RegistryError)?;
+                    use crate::dimension::DimensionRegistryError;
+                    self.registry.add_base_dimension(name).map_err(|err| {
+                        TypeCheckError::from(DimensionRegistryError {
+                            span: *name_span,
+                            err,
+                        })
+                    })?;
                 }
                 typed_ast::Statement::DefineDimension(name, dexprs.clone())
             }
@@ -1760,6 +1871,7 @@ impl TypeChecker {
             ast::Statement::DefineStruct {
                 struct_name_span,
                 struct_name,
+                type_parameters,
                 fields,
             } => {
                 self.type_namespace
@@ -1772,6 +1884,34 @@ impl TypeChecker {
 
                 let mut seen_fields = HashMap::new();
 
+                let mut typechecker_struct = self.clone();
+
+                for (span, type_parameter, bound) in type_parameters {
+                    if typechecker_struct
+                        .type_namespace
+                        .has_identifier(type_parameter)
+                    {
+                        return Err(Box::new(TypeCheckError::TypeParameterNameClash(
+                            *span,
+                            type_parameter.to_string(),
+                        )));
+                    }
+
+                    typechecker_struct
+                        .type_namespace
+                        .add_identifier(
+                            type_parameter.to_compact_string(),
+                            *span,
+                            "type parameter".to_compact_string(),
+                        )
+                        .ok();
+
+                    typechecker_struct
+                        .registry
+                        .introduced_type_parameters
+                        .push((*span, type_parameter.to_compact_string(), bound.clone()));
+                }
+
                 for (span, field, _) in fields {
                     if let Some(other_span) = seen_fields.get(field) {
                         return Err(Box::new(TypeCheckError::DuplicateFieldInStructDefinition(
@@ -1787,12 +1927,20 @@ impl TypeChecker {
                 let struct_info = StructInfo {
                     definition_span: *struct_name_span,
                     name: struct_name.to_compact_string(),
+                    kind: StructKind::Definition(
+                        type_parameters
+                            .iter()
+                            .map(|(span, name, bound)| {
+                                (*span, (*name).to_compact_string(), bound.clone())
+                            })
+                            .collect(),
+                    ),
                     fields: fields
                         .iter()
                         .map(|(span, name, type_)| {
                             Ok((
                                 name.to_compact_string(),
-                                (*span, self.type_from_annotation(type_)?),
+                                (*span, typechecker_struct.type_from_annotation(type_)?),
                             ))
                         })
                         .collect::<Result<_>>()?,
