@@ -249,8 +249,10 @@ impl DType {
                 DTypeFactor::TVar(TypeVariable::Named(name)) => {
                     factors.push(BaseRepresentationFactor(name.clone(), *n));
                 }
-                DTypeFactor::TVar(TypeVariable::Quantified(_)) => {
-                    unreachable!("Unexpected quantified type")
+                DTypeFactor::TVar(TypeVariable::Quantified(id)) => {
+                    // Quantified type variables can appear during constraint solving
+                    // before they're fully resolved
+                    factors.push(BaseRepresentationFactor(format!("?{id}").into(), *n));
                 }
                 DTypeFactor::TPar(name) => {
                     factors.push(BaseRepresentationFactor(name.clone(), *n));
@@ -283,10 +285,20 @@ impl From<BaseRepresentation> for DType {
     }
 }
 
+/// Represents whether a struct is a generic definition or a concrete instance
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub enum StructKind {
+    /// Generic struct definition with type parameters (e.g., `struct Vec<D: Dim>`)
+    Definition(Vec<(Span, CompactString, Option<TypeParameterBound>)>),
+    /// Instantiated struct with concrete type arguments (e.g., `Vec<Length>`)
+    Instance(Vec<Type>),
+}
+
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct StructInfo {
     pub definition_span: Span,
     pub name: CompactString,
+    pub kind: StructKind,
     pub fields: IndexMap<CompactString, (Span, Type)>,
 }
 
@@ -323,11 +335,20 @@ impl std::fmt::Display for Type {
                 )
             }
             Type::Struct(info) => {
-                let StructInfo { name, fields, .. } = info.as_ref();
+                write!(f, "{}", info.name)?;
+                if let StructKind::Instance(type_args) = &info.kind {
+                    if !type_args.is_empty() {
+                        write!(
+                            f,
+                            "<{}>",
+                            type_args.iter().map(|t| t.to_string()).join(", ")
+                        )?;
+                    }
+                }
                 write!(
                     f,
-                    "{name} {{{}}}",
-                    fields
+                    " {{{}}}",
+                    info.fields
                         .iter()
                         .map(|(n, (_, t))| n.to_string() + ": " + &t.to_string())
                         .join(", ")
@@ -365,7 +386,22 @@ impl PrettyPrint for Type {
                     + return_type.pretty_print()
                     + m::operator("]")
             }
-            Type::Struct(info) => m::type_identifier(info.name.clone()),
+            Type::Struct(info) => {
+                let mut markup = m::type_identifier(info.name.clone());
+                if let StructKind::Instance(type_args) = &info.kind {
+                    if !type_args.is_empty() {
+                        markup = markup + m::operator("<");
+                        markup = markup
+                            + Itertools::intersperse(
+                                type_args.iter().map(|t| t.pretty_print()),
+                                m::operator(",") + m::space(),
+                            )
+                            .sum();
+                        markup = markup + m::operator(">");
+                    }
+                }
+                markup
+            }
             Type::List(element_type) => {
                 m::type_identifier("List")
                     + m::operator("<")
@@ -380,6 +416,28 @@ impl Type {
     pub fn to_readable_type(&self, registry: &DimensionRegistry) -> Markup {
         match self {
             Type::Dimension(d) => d.to_readable_type(registry),
+            Type::Struct(info) => {
+                let mut markup = m::type_identifier(info.name.clone());
+                if let StructKind::Instance(type_args) = &info.kind {
+                    if !type_args.is_empty() {
+                        markup = markup + m::operator("<");
+                        markup = markup
+                            + Itertools::intersperse(
+                                type_args.iter().map(|t| t.to_readable_type(registry)),
+                                m::operator(",") + m::space(),
+                            )
+                            .sum();
+                        markup = markup + m::operator(">");
+                    }
+                }
+                markup
+            }
+            Type::List(element_type) => {
+                m::type_identifier("List")
+                    + m::operator("<")
+                    + element_type.to_readable_type(registry)
+                    + m::operator(">")
+            }
             _ => self.pretty_print(),
         }
     }
@@ -451,7 +509,33 @@ impl Type {
                     .collect(),
                 Box::new(return_type.instantiate(type_variables)),
             ),
-            t @ Type::Struct(_) => t.clone(),
+            Type::Struct(info) => {
+                let instantiated_fields = info
+                    .fields
+                    .iter()
+                    .map(|(name, (span, field_type))| {
+                        (
+                            name.clone(),
+                            (*span, field_type.instantiate(type_variables)),
+                        )
+                    })
+                    .collect();
+                let instantiated_kind = match &info.kind {
+                    StructKind::Definition(params) => StructKind::Definition(params.clone()),
+                    StructKind::Instance(type_args) => StructKind::Instance(
+                        type_args
+                            .iter()
+                            .map(|t| t.instantiate(type_variables))
+                            .collect(),
+                    ),
+                };
+                Type::Struct(Box::new(StructInfo {
+                    definition_span: info.definition_span,
+                    name: info.name.clone(),
+                    kind: instantiated_kind,
+                    fields: instantiated_fields,
+                }))
+            }
             Type::List(element_type) => {
                 Type::List(Box::new(element_type.instantiate(type_variables)))
             }
@@ -623,6 +707,7 @@ pub enum Statement<'a> {
     DefineDimension(&'a str, Vec<TypeExpression>),
     DefineBaseUnit(
         &'a str,
+        Span,
         Vec<Decorator<'a>>,
         Option<TypeAnnotation>,
         TypeScheme,
@@ -723,7 +808,7 @@ impl Statement<'_> {
                 }
             }
             Statement::DefineDimension(_, _) => {}
-            Statement::DefineBaseUnit(_, _, _, _) => {}
+            Statement::DefineBaseUnit(_, _, _, _, _) => {}
             Statement::DefineDerivedUnit(_, _, _, _, type_annotation, type_, readable_type) => {
                 *readable_type =
                     Self::create_readable_type(registry, type_, type_annotation, false);
@@ -1067,7 +1152,7 @@ impl PrettyPrint for Statement<'_> {
                     )
                     .sum()
             }
-            Statement::DefineBaseUnit(identifier, decorators, annotation, type_) => {
+            Statement::DefineBaseUnit(identifier, _, decorators, annotation, type_) => {
                 decorator_markup(decorators)
                     + m::keyword("unit")
                     + m::space()
@@ -1076,7 +1161,7 @@ impl PrettyPrint for Statement<'_> {
                     + m::space()
                     + annotation
                         .as_ref()
-                        .map(|a| a.pretty_print())
+                        .map(|a: &TypeAnnotation| a.pretty_print())
                         .unwrap_or(type_.pretty_print())
             }
             Statement::DefineDerivedUnit(
