@@ -722,7 +722,10 @@ impl Vm {
                         Op::Divide => Ok(lhs
                             .checked_div(rhs)
                             .ok_or_else(|| self.runtime_error(RuntimeErrorKind::DivisionByZero))?),
-                        Op::Power => lhs.power(rhs),
+                        Op::Power => Ok(lhs
+                            .checked_power(rhs)
+                            .map_err(|e| self.runtime_error(RuntimeErrorKind::QuantityError(e)))?
+                            .ok_or_else(|| self.runtime_error(RuntimeErrorKind::DivisionByZero))?),
                         // If the user specifically converted the type of a unit, we should NOT simplify this value
                         // before any operations are applied to it
                         Op::ConvertTo => lhs.convert_to(rhs.unit()).map(Quantity::no_simplify),
